local beql = LibStub("AceAddon-3.0"):NewAddon("beql", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0")
beqlQ = LibStub("LibQuixote-2.0")
local L = LibStub("AceLocale-3.0"):GetLocale("beql")


local VERSION = GetAddOnMetadata("beql", "Version")
beql.revision = VERSION
beql.versionstring = VERSION
beql.beta = 0
beql.cHeader = {}

--[===[@debug@
beql.debug = true
SHOW_QUEST_OBJECTIVES_ON_MAP = "1"
--@end-debug@]===]

beql.FrameCache = {}

-- create Buttoncache
beql.FrameCache.Buttons = UIFrameCache:New("BUTTON", "beqlLinkButton", UIParent, "WatchFrameLinkButtonTemplate")
beql.FrameCache.QuestLinkButtons = {}
beql.FrameCache.QuestLinkButtonIndex = 1
beql.FrameCache.AchievementLinkButtons = {}
beql.FrameCache.AchievementLinkButtonIndex = 1

-- create Linecache
beql.FrameCache.Lines = UIFrameCache:New("FRAME", "beqlLine", UIParent, "WatchFrameLineTemplate");
beql.FrameCache.QuestLines = {}
beql.FrameCache.QuestLineIndex = 1
beql.FrameCache.AchievementLines = {}
beql.FrameCache.AchievementLineIndex = 1

-- Debug Function
-- #NODOC
function beql:Print(text)
    DEFAULT_CHAT_FRAME:AddMessage(tostring(text))
end

function beql:debugprint(text)
	if beql.debug then
		DEFAULT_CHAT_FRAME:AddMessage("bEQL: " .. tostring(text))
	end
end

-- Addon functions

-- ace2 -> ace3 compatibility functions
if not beql.IsActive then
	function beql:IsActive()
		return beql:IsEnabled();
	end
end

if not beql.ToggleActive then
	function beql.ToggleActive()
		if beql:IsEnabled() then
			beql:Disable()
			beql.db.profile.AddonStatus = false
		else
			beql:Enable()
			beql.db.profile.AddonStatus = true
		end
	end
end

-- Init beql addon
function beql:OnInitialize()
	hooksecurefunc("WatchFrame_ClearDisplay", function()
		-- WatchFrame_ClearDisplay doesn't hide QuestItemButtons, we must fix it
		for i=1, 25 do
			local itembutton = _G["WatchFrameItem" .. i]
			if itembutton then
				itembutton:Hide()
			else
				break;
			end
		end
	end)
	
	-- hook loading watchframeline load template
	hooksecurefunc("WatchFrameLineTemplate_OnLoad", function(line)
		local linename = line:GetName()
		-- we reference this items, to access them faster
		line.iconframe = CreateFrame("Frame", linename .. "IconFrame", line)
		line.icon = _G[linename.."Icon"]
		line.icon:Show()
		line.iconborder =	_G[linename.."Border"]
		line.iconborder:Show()
		line.icon:SetParent(line.iconframe)
		line.iconborder:SetParent(line.iconframe)
		line.iconframe:SetAllPoints(line.icon)
		line.iconframe:Hide()
		
		line.mapButton:SetScript("OnClick", function(mapbutton)
			WorldMap_OpenToQuest(mapbutton.questID);
		end)
		
		line.Reset = function(line)
			-- org Blizzard function
			line:ClearAllPoints();
			line.text:SetWidth(0);
			line.text:ClearAllPoints();
			line.text:SetPoint("TOPLEFT");
			line.text:SetPoint("BOTTOMRIGHT");
			line.text:SetText("");
			line.text:SetTextColor(1, 1, 1);
			line.text:Show();
			line.statusBar:Hide();
			line.mapButton:Hide();
			line:SetHeight(WATCHFRAME_LINEHEIGHT);
			line.criteriaID = nil;
			
			-- mod start
			line.iconframe:Hide();
			line.iconframe.aID = nil;
			line.qID = nil;
			line.aID = nil;
			line.subaID = nil;
		end

	end)

	beql:debugprint("Init beql Addon")
	if not beql.db then
		beql:InitDB()
	end

	beql:BlizzardSetupOptions()
	beql:RegisterChatCommand("beql", function() beql:OpenBlizzardOption("main") end)

	
	beql.QuestItems = 0
	
	beql:SetEnabledState(beql.db.profile.AddonStatus)
	if not beql.db.profile.debug then
		beql.db.profile.debug = true
	end

	
	beql:RawHook("SendChatMessage", true)
	beql:debugprint("Init beql Addon - done")
end

-- Hook Oldstyle to prevent Quest Gained Bug
beqlQ.Hook_QUEST_LOG_UPDATE = beqlQ.frame.QUEST_LOG_UPDATE

function beqlQ.frame:QUEST_LOG_UPDATE()

	-- manage quests
	if beql:IsActive() then
		beql:ManageQuests()
	end
	
	-- run org script
	beqlQ.Hook_QUEST_LOG_UPDATE()

end

	
function beql:OnEnable(...)
	if Expo then 
		self.debugFrame = Expo:ReturnDebugFrame() 
	end
	
	-- Check for other addons
	beql:Compatibility()
	
	-- Init Questlog
	beql:InitQuestLog()
	beql:ToggleQuestlogStyle(beql.db.profile.simplequestlog)
	beql:SetMovableQuestlog(self.db.profile.locked)
	
	-- Load Questtracker
	if not beql.db.profile.disabled.tracker and not beql.db.profile.disabledtracker then
		beql:ToggleQuestTracker(beql.db.profile.disabledtracker)
		beql:SetMovableQuestTracker(beql.db.profile.locked)
	end


	-- Init Tooltip
	beql:InitTooltip()
	
	-- Load Achievement Tracker
	--beql.db.profile.atracker.Enable = false -- 4 debug
	--beql.db.profile.atracker.savelast = false -- 4 debug

	-- Achievements
	
	beql:ToggleAchievementTrackerStatus()


	if not beql.db.profile.revision or beql.db.profile.revision ~= beql.revision then
		-- make a infobox
		if beql.beta == 1 then
			--message("This is a beta release.\n Please report bugs on curseforge.\n Angelike")
		else
			--message("bEQL has got an update, please check your settings")
		end
			
		beql.db.profile.revision = beql.revision
	end
	
	--
	-- Hooks
	--
	
	beql:SecureHook("WatchFrame_Update", beql.Hooks_WatchFrame_Update)
	
	--
	-- Callbacks
	--
	
	beqlQ.RegisterCallback(beql, "Objective_Update", beql.Quixote_Objective_Update)
	beqlQ.RegisterCallback(beql, "Quest_Complete", beql.Quixote_Quest_Complete)
	
--[[
	-- Group handle
	-- unfinished business
		beqlQ.RegisterCallback(beql, "Party_Quest_Gained")
		beqlQ.RegisterCallback(beql, "Party_Quest_Complete")
		beqlQ.RegisterCallback(beql, "Party_Quest_Failed")
		beqlQ.RegisterCallback(beql, "Party_Quest_Lost")
		beqlQ.RegisterCallback(beql, "Party_Objective_Update")
		beqlQ.RegisterCallback(beql, "Party_Update")
		beqlQ.RegisterCallback(beql, "Sync_Finished")
		beql:RegisterEvent("CHAT_MSG_ADDON")
		--]]
end

function beql:OnDisable()
	-- backup flags
	local questlogstatus = beql.db.profile.simplequestlog
	local questtrackerstatus = beql.db.profile.disabledtracker
	local achievementtrackerstatus = beql.db.profile.atracker.Enable
	
	-- disable Questlog
	beql:ToggleQuestlogStyle(true)
	
	-- disable Tracker
	beql:ToggleQuestTracker(true)
	
	-- disable Achievementtracker
--	beql:ToggleAchievementTracker(false)

	-- restore old flags
	beql.db.profile.simplequestlog = questlogstatus
	beql.db.profile.disabledtracker = questtrackerstatus
	beql.db.profile.atracker.Enable = achievementtrackerstatus
	
	-- Questlog
	QuestLogTitleText:SetText(QUEST_LOG)
	beql:SetQuestLogFontSize(13)
	QuestLogFrame:SetAlpha(1)
	QuestLogFrame:SetScale(1)
	QuestLogFrame:RegisterForDrag(0)
	QuestLogFrame:ClearAllPoints()
	QuestLogFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 0,-104)
	
	
	beql.ConfigButton:Hide()
	
	beql:UnhookAll();
	
	-- Update Frames
	WatchFrame_Update()
	QuestLog_Update()
	
	
	beqlQ.UnregisterCallback(beql, "Objective_Update");
	beqlQ.UnregisterCallback(beql, "Quest_Complete");

--	QuestLogFrame:TriggerEvent("QUESTLOG_UPDATE")
end

-- Internal for setting default values for the variables
-- #NODOC
function beql:InitDB()
	local defaults = {
	profile = {
		AddonStatus = true,
		disabled = {},
		Color = 
		{
			BackgroundColor = {
				r = TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,
				g = TOOLTIP_DEFAULT_BACKGROUND_COLOR.g,
				b = TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,
			},
			BackgroundCornerColor = {
				r = TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,
				g = TOOLTIP_DEFAULT_BACKGROUND_COLOR.g,
				b = TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,			
			},
			Zone = {
				r = 1,
				g = 1,
				b = 1,
			},
			HeaderComplete = {
				r = NORMAL_FONT_COLOR.r,
				g = NORMAL_FONT_COLOR.g,
				b = NORMAL_FONT_COLOR.b,
			},
			HeaderEmpty = {
				r = 0.75,
				g = 0.61,
				b = 0,
			},
			HeaderFailed = {
				r = 1,
				g = 0,
				b = 0,
			},
			ObjectiveEmpty = {
				r = 0.8,
				g = 0.8,
				b = 0.8,
			},
			ObjectiveComplete = {
				r = HIGHLIGHT_FONT_COLOR.r,
				g = HIGHLIGHT_FONT_COLOR.g,
				b = HIGHLIGHT_FONT_COLOR.b,
			},
			TooltipQuestTitle = {
				r = 0.75,
				g = 0.61,
				b = 0,
			},
			TooltipDesc = {
				r = 1.0,
				g = 0.8,
				b = 0,	
			},
			TooltipPartyQuixote = {
				r = 0.4,
				g = 0.4,
				b = 1,
			},
			TooltipPartyNonQuixote = {
				r = 1,
				g = 0.4,
				b = 0.4,
			},
			TooltipPartyObj = {
				r = 0.8,
				g = 0.8,
				b = 0.8,
			},
			TooltipPartyObjComp = {
				r = HIGHLIGHT_FONT_COLOR.r,
				g = HIGHLIGHT_FONT_COLOR.g,
				b = HIGHLIGHT_FONT_COLOR.b,
			},
			ColorMobTooltip = {
				r = 0.8,
				g = 0.8,
				b = 0.8,
			},
			ColorItemTooltip = {
				r = 0.8,
				g = 0.8,
				b = 0.8,
			},
		},
		
-- QuestLog
		locked = true,
		alwaysminimize = false,
		alwaysmaximize = false,
		showlevel = true,
		InfoOnQuestCompletion = false,
		autocomplete = false,
		simplequestlog = false,
		QuestLogAlpha = 1,
		QuestLogScale = 1,
		forcemove = false,
		
-- QuestTracker
		lockedtracker = false,
		disabledtracker = false,
		ShowTrackerHeader = false,
		TrackerFontHeight = 12,
		CustomBackgroundColor = false,
		BackgroundAlpha = false,		
		ShowZonesInTracker = false,
		CustomZoneColor = false,
		CustomObjetiveColor = false,
		RemoveFinished = true,
		MinimizeFinished = false,
		HideCompletedOnly = false,
		CustomHeaderColor = false,
		HeaderQuestLevelColor = false,
		ShowObjectiveMarkers = true,
		UseTrackerListing = false,
		TrackerList = 0,
		TrackerSymbol = 0,
		SortTrackerItems = true,
		AddNew = true,
		AddUntracked = true,
		FadeColor = false,
		QuestTrackerAlpha = 1,
		TrackerAutoResize = true,
		TrackerFixedWidth = 250,
		
-- Tooltip
		tooltipmob = true,
		tooltipitem = true,
		activetracker = true,
		activetrackerdesc = true,		
		activetrackerleft = false,
		activetrackerright = false,
		activetrackerparty = false,
		AnnounceQuest = false,
		InfoSound = "Sound\\Interface\\GnomeExploration.wav",
		QuestLogFontSize = 12,
		TooltipTitleDiff = true,
		TooltipObjFade = true,
		TooltipItemFade = true,

-- Fubar
		fubar = {
			HidePlugin = false,
			ShowIcon = true,
			ShowText = true,
			ShowColoredText = true,
			ShowTooltip = true,
			FubarPosition = "MINIMAP",
			MinimapPosition = 238,
		},
-- Achievementtracker
		atracker = {
			-- mouse events
			showtooltip = true,
			showtooltiphint = false,
			clickleft = true,
			clickright = true,
			
			disableButtons = false,
			ManageLines = true,
			NumLines = 20,
			savelast	= true,
			Enable	= true,
			Locked	= true,
			headline	= true,
			alpha		= 1,
			scale 	= 0.9,
			RemoveFinished 	= true,
			showbg		= false,
			ownbgcolor	= false,
			BgColor 	= {
				r	= TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,
				g	= TOOLTIP_DEFAULT_BACKGROUND_COLOR.g,
				b	= TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,
			},
			BgCornerColor 	= {
				r	= TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,
				g	= TOOLTIP_DEFAULT_BACKGROUND_COLOR.g,
				b	= TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,			
			},
		},
	},
	char = {
		QuestDB = {},
		QuestWatches = {},
		QuestCompleted = {},
		saved = {},
		TrackerQuests = {},
		AchievementTracker = {
			Achievements = {},
			minimized = false,
			posy = 0,
			posx = 0,
		},
	},
	realm = {
		QuestDB = {},
	},
	}
	beql.db = LibStub("AceDB-3.0"):New("beqlDBnew", defaults, true)
end

--------------------------------------------
-- Arguments:
-- number - quest id
-- string - quest name
--
-- Notes:
-- Prints the Quest Name/Status and Objectives to the currently opened chatbox
--
-- Returns:
-- * nothing
--------------------------------------------
function beql:AddQuestStatusToChatFrame(questIndex)
	local chatFrame = DEFAULT_CHAT_FRAME
	local chatType = chatFrame.editBox:GetAttribute("chatType")
	local oText, oType, oNumP, oNumN, oComp
	local text
	if chatType == "WHISPER" then
		chatType = "SAY"
	end
	local qID, qIndex , qTitle, qLvl, qTag, qObj, qStat, qRec, qdaily, qZone = beqlQ:GetQuestById(questIndex) 
	if qObj and qObj > 0 then
		SendChatMessage("["..qLvl..string.upper(beqlQ:GetShortTagForQuest(qID)).."] "..qTitle, chatType)
		local _, qObjectives = beqlQ:IterateObjectivesForQuest(qID)
		for oText, oItems in pairs(qObjectives) do
			text = ""
			oComp = false;
			oNumP, oNumN, oType = unpack(oItems)
			if oNumN == nil then oNumN = 1 end
			if oNumP == nil then oNumP = 1 end
			if oText == nil then oText = "" end
			if oNumP == oNumN then
				oComp = true
			end
			
			text = oText
			
			if oComp then
				text = text.." "..L["(Done)"]
			else
				text = text..": "..oNumP.."/"..oNumN
			end
			if text and strlen(text) > 0 then
				SendChatMessage(" - "..text, chatType)
			end
		end

--[[		for i=1, qObj do
			text = ""
			oText, oType, oNumP, oNumN, oComp = beqlQ:GetQuestStatusById(questIndex, i)
			text = oText
			if finished then
				text = text.." "..L["(Done)"]
			else
				text = text..": "..oNumP.."/"..oNumN
			end
			if text and strlen(text) > 0 then
				SendChatMessage(" - "..text, chatType)
			end
		end--]]
	else
		chatFrame:AddMessage(L["No Objectives!"])
	end
	chatFrame.editBox:Hide()
end

--------------------------------------------
-- Notes:
-- Sorts the Tracked quests
--
-- Returns:
-- * nothing
--------------------------------------------
function beql:SortWatchedQuests()
	if beql.db.profile.ShowZonesInTracker and beql.db.profile.SortTrackerItems then
		table.sort(beql.db.char.QuestWatches)
	else
		if beql.db.profile.SortTrackerItems then
			table.sort(beql.db.char.QuestWatches, SortCompare)
		end
	end
end

function SortCompare(first, second)
	local temp = string.gsub(first, ".+,%d+,[%[%d+%+*%]]*", "")
	local temp2 = string.gsub(second, ".+,%d+,[%[%d+%+*%]]*", "")
	if temp < temp2  then
		return true
	end
	return false
end

--------------------------------------------
-- Notes:
-- Manage Tracked Quest list
--
-- Returns:
-- * nothing
--------------------------------------------
function beql:ManageQuests()
	local tempHeaderList = {}
	local tempQuestList = {}
	local numEntries = GetNumQuestLogEntries()
	local questLogTitleText, level, questTag, isHeader, isCollapsed, isComplete
	local currentHeader = nil
	local temp, _
	
	local oldTitle

	-- for new quests in questlog
	local oldquestids = beql.db.char.questids 
	beql.db.char.questids = {}
	beql.db.char.newquestid = 0
	
	if not oldquestids then
		oldquestids = {}
	end

	
	if numEntries ~= 0 then
		for j=numEntries, 1, -1 do
			questLogTitleText, _, _, _, isHeader, isCollapsed = GetQuestLogTitle(j)
			if (isHeader and isCollapsed) then
--				print("epxpanding " .. questLogTitleText)
				tempHeaderList[j] = 1
				ExpandQuestHeader(j)
			end
		end

		numEntries = GetNumQuestLogEntries()
	
		for j=1, numEntries, 1 do

			-- rescan Questid's
			if GetQuestLink(j) then
				local id = tonumber(string.match(GetQuestLink(j), 'quest:(%d+)'))
				table.insert(beql.db.char.questids, id, id)
				if not oldquestids[id] then
					beql.db.char.newquestid = id
				end
					
			end

			questLogTitleText, level, questTag, _, isHeader, isCollapsed, isComplete = GetQuestLogTitle(j)
			oldTitle = self.hooks.GetQuestLogTitle(j)
			if isHeader then
				currentHeader = questLogTitleText
			else
--				print("scanning " .. questLogTitleText)
				temp = currentHeader..","..level..","..questLogTitleText
				table.insert(tempQuestList, temp)
--				if beql.TrackerQueue and beql.TrackerQueue == oldTitle then
--					table.insert(beql.db.char.QuestWatches, temp)
--					beql.TrackerQueue = nil
--				end
			end
		end	
	
		for j=1, numEntries, 1 do
			if(tempHeaderList[j] == 1) then
				CollapseQuestHeader(j)
			end
		end
	
		local numWatches = table.getn(beql.db.char.QuestWatches)
		local numEntries = table.getn(tempQuestList)
		local found = false
		for i=numWatches, 1, -1 do
			found = false
			for j=0, numEntries, 1 do
				if beql.db.char.QuestWatches[i] == tempQuestList[j] then
					found = true
					break
				end
			end
			if not found then
				table.remove(beql.db.char.QuestWatches, i)
			end
		end	
	end
	beql:SortWatchedQuests()
end

--------------------------------------------
-- Arguments:
-- table - Color1
-- table - Color2
-- number - Have
-- number - Needed
--
-- Notes:
-- Fades Colors between Color1 and Color2 depending on how much percent has Have from Needed
--
-- Returns:
-- * table - Color Faded
--------------------------------------------
function beql:FadeColors(tempColor, tempColor2, done, obj)
	local color = {r=0, g=0, b=0}
	local multiplier

	multiplier = (done / obj)
	color.r = tempColor.r + ((tempColor2.r - tempColor.r)*multiplier)
	color.g = tempColor.g + ((tempColor2.g - tempColor.g)*multiplier)
	color.b = tempColor.b + ((tempColor2.b - tempColor.b)*multiplier)

	return color

end

--------------------------------------------
-- Arguments:
-- number - Decimal nr
-- number - Length of number
--
-- Notes:
-- Converts a Decimal Number to Hexadecimal
--
-- Returns:
-- string - Hexadecimal number
--------------------------------------------
function beql:decToHex(Dec, Length)
	local B, K, Hex, I, D = 16, "0123456789ABCDEF", "", 0;
	while Dec>0 do
		I=I+1;
		Dec, D = math.floor(Dec/B), math.fmod(Dec,B)+1;
		Hex=string.sub(K,D,D)..Hex;
	end
	if( (Length ~= nil) and (string.len(Hex) < Length) ) then
		local temp, i = Length-string.len(Hex), 1;
		for i=1, temp, 1 do
			Hex = "0"..Hex;
		end
	end
	return Hex;
end

--------------------------------------------
-- Arguments:
-- string - text
-- number - Red Component
-- number - Green Component
-- number - Blue Component
--
-- Notes:
-- Returns a WoW-style colored text
--
-- Returns:
-- string - Text colored with WoW control characters
--------------------------------------------
function beql:ColorText(t, r, g, b)
	if ( t == nil ) then t = ""; end
	if ( r == nil ) then r = 0.0; end
	if ( g == nil ) then g = 0.0; end
	if ( b == nil ) then b = 0.0; end
	return "|CFF"..beql:decToHex(r*255, 2)..beql:decToHex(g*255, 2)..beql:decToHex(b*255, 2)..t.."|r";
end

--------------------------------------------
-- Arguments:
-- string - text
--
-- Notes:
-- Prints all Points of a Region
--
-- Returns:
-- nothing

-- function to ge points of an region
-- #NODOC
function beql:printpoints(tmp)
	local region
	if type(tmp) == 'string' then 
		region = getglobal(tmp)
	else
		region = tmp
	end
	local i
	
	print("Framepoints for " .. tostring(region:GetName()))
	for i=1, region:GetNumPoints(),1 do 
		local point, relativeTo, relativePoint, xOfs, yOfs = region:GetPoint(i)
		print(point .. " " .. relativeTo:GetName() .. " " .. relativePoint .. " " ..  xOfs .." ".. yOfs)
	end
end

function beql:round(input, places)
	if not places then
			places = 0
	end
	if type(input) == "number" and type(places) == "number" then
			local pow = 1
			for i = 1, ceil(places) do
					pow = pow * 10
			end
			return floor(input * pow + 0.5) / pow
	end
end


-- 
--- Party Functions
--

function beql:Quixote_Objective_Update(title, uid, objective, had, numGot, numNeeded, qtype)
	local _, id, title, level, tag, NumQuestLeaderBoard, complete, group, daily, zone = beqlQ:GetQuestByUid(uid)
	
	-- add unwatched quest
	if beql.db.profile.AddUntracked == true and (not IsQuestWatched(id)) then
		AddQuestWatch(id)
	end
	
	-- print progress to party
	if beql.db.profile.AnnounceQuest then
		local text = ""
		local objText = ""
		local qstatus
		
		if qtype == "item" or qtype == "monster" or qtype == "reputation" then
			objText = objective..": "..numGot.."/"..numNeeded
		end		
--		text = description..objText
		beql:Party_Status2Chat(objText)

--		if numGot == numNeeded and (complete == 1) then
--			beql:Party_Status2Chat(title .." "..L["Completed!"])
--		end		
	end
end

function beql:Quixote_Quest_Complete(questname, uid)
	beql:Party_Status2Chat(questname .." "..L["Completed!"])

	if beql.db.profile.InfoOnQuestCompletion then
		UIErrorsFrame:AddMessage(questname.." "..L["Completed!"], 1.0, 0.8, 0.0, 1.0, UIERRORS_HOLD_TIME)
		if beql.db.profile.InfoSound and beql.db.profile.InfoSound ~= "" then 
			PlaySoundFile(beql.db.profile.InfoSound)
		end
	end
end

function beql:Party_Status2Chat(msg)
	if beql.db.profile.AnnounceQuest and (GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0) then
		local chatType = "PARTY"
		
		if GetNumRaidMembers() > 0 then
			chatType = "RAID"
		end
		
		SendChatMessage(msg, chatType)
	end
end

function beql:Party_Quest_Gained (event)
	print(event)
	beql:Party_frame_Update()
end

function beql:Party_Quest_Complete (event)
	print(event)
	beql:Party_frame_Update()
end

function beql:Party_Quest_Failed (event)
	print(event)
	beql:Party_frame_Update()
end

function beql:Party_Quest_Lost (event)
	print(event)
	beql:Party_frame_Update()
end

function beql:Party_Objective_Update (event)
	print(event)
	beql:Party_frame_Update()
end

function beql:Party_Update (event)
	print(event)
	beql:Party_frame_Update()
end

function beql:Party_frame_Update()
	if not QuestLogPartyQuestFrame.buttons then return end
	
	local questids = {}
	local NumButtons = #QuestLogPartyQuestFrame.buttons
	local buttonheigth = QuestLogPartyQuestFrame.buttons[1]:GetHeight()
	local ScrollOffset = HybridScrollFrame_GetOffset(QuestLogPartyQuestFrame)
	for k in pairs(beqlQ.UID_to_name) do
		table.insert(questids,k)
	end
	
	local j = -ScrollOffset
	local questTag, textcolor
--	for l=1,4 do
	
	for k, v in pairs(beqlQ.party_quests) do
		j = j + 1
		if j <= NumButtons and j > 0 then
			QuestLogPartyQuestFrame.buttons[j]:SetText(k)
			QuestLogPartyQuestFrame.buttons[j]:SetID(0)
			QuestLogPartyQuestFrame.buttons[j].normalText:SetTextColor(1,1,1)
			QuestLogPartyQuestFrame.buttons[j]:Show()
			QuestLogPartyQuestFrame.buttons[j]:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up");
		end
		for k, quest in pairs(v) do
			j = j + 1
			if j <= NumButtons and j > 0 then
				QuestLogPartyQuestFrame.buttons[j]:SetText(" " .. beqlQ.UID_to_name[k])
				QuestLogPartyQuestFrame.buttons[j].normalText:SetTextColor(1,0.82,0)
				QuestLogPartyQuestFrame.buttons[j]:SetID(k)
				QuestLogPartyQuestFrame.buttons[j]:Show()
				QuestLogPartyQuestFrame.buttons[j]:SetNormalTexture("");
				
				
				
				
				-- tag and color
				-- figure out which tag to show, if any
				if ( quest.completed and quest.completed < 0 ) then
					questTag = FAILED;
					textcolor = QuestDifficultyColors["impossible"]
				elseif ( quest.completed and quest.completed > 0 ) then
					questTag = COMPLETE;
					textcolor = QuestDifficultyColors["standard"]
				elseif ( not quest.completed or quest.completed == 0 ) then
					questTag = nil
					textcolor = QuestDifficultyColors["difficult"]
				end
				if ( questTag ) then
					QuestLogPartyQuestFrame.buttons[j].tag:SetText("("..questTag..")");
					QuestLogPartyQuestFrame.buttons[j].tag:SetTextColor(textcolor.r or 0, textcolor.g or 0, textcolor.b or 0);
					QuestLogPartyQuestFrame.buttons[j].tag:Show();
				else
					QuestLogPartyQuestFrame.buttons[j].tag:Hide();
				end
				
				QuestLogPartyQuestFrame.buttons[j].normalText:SetTextColor(textcolor.r or 0, textcolor.g or 0, textcolor.b or 0);
				-- fit text and color
				--QuestLogTitleButton_Resize(QuestLogPartyQuestFrame.buttons[j]);
			end
		end
	end
	
--	end
	
	j = j + ScrollOffset
	HybridScrollFrame_Update(QuestLogPartyQuestFrame, j * buttonheigth, #QuestLogPartyQuestFrame.buttons * buttonheigth);
end

function beql:Sync_Finished(event, char)
	print("sync Finished with " .. char)
	if GetBattlefieldInstanceRunTime() ~= 0 then 
		-- we are in a battlefield, do nothing, just return
		if strmatch(char, '(\-)') then
			-- player is not from our realm
			return
		end
	end
	
	local questids
	for k, v in pairs (beqlQ.party_quests[char]) do
		if not v.completed then
			if not questids then
				questids = "qstatusrequest_" .. tostring(k)
			else
				questids = questids .. ":" .. tostring(k)
			end
		end
	end

	if questids then
		beql:SendAddonMessage(questids, "WHISPER", char,"BULK")
	end
end

local ChatThrottleLib = ChatThrottleLib
function beql:SendAddonMessage(contents, distribution, target, priority)
	if ChatThrottleLib then
		ChatThrottleLib:SendAddonMessage(priority or "NORMAL", "beql", contents, distribution or "PARTY", target);
	end
end

function beql:CHAT_MSG_ADDON(event, prefix, msg, distribution, sender)
	if prefix ~= "beql" then 
		-- not a massage for me, exit
		return 
	end
	
	comtype, commsg = strsplit("_",msg,2)
	
	
	-- return the complete status about a quest
	if comtype == "qstatusrequest" then
		local questids
		questids = {strsplit(":",commsg)}
		local answer
		local uid
		for k, uid in pairs(questids) do
			uid = tonumber(uid)
			if not answer then
				answer = "qstatusanswer_" .. tostring(uid) ..":"..tostring(select(7,beqlQ:GetQuestByUid(uid)) or 0) -- to overwrite nil for unfinished
			else
				answer = answer .. ";" .. tostring(uid) ..":"..tostring(select(7,beqlQ:GetQuestByUid(uid)) or 0) -- to overwrite nil for unfinished
			end
		end
		
		if answer then
			beql:SendAddonMessage(answer, distribution, sender, "BULK")
		end
		return
	
	
	-- write completition status in db
	elseif comtype == "qstatusanswer" then
		-- data is "uid:status;uid:status;..."
		if not beqlQ.party_quests[sender] then 
			-- player not found in db
			assert(false, "player \"" .. sender .. "\" not found!") 
		end
		
		-- split answer in uid:status pices
		local questids = {strsplit(";",commsg)} 
		local uid, qstatus
		for k, quest in pairs(questids) do
			uid, qstatus = strsplit(":",quest)
			-- writ data in DB
			if not beqlQ.party_quests[sender][tonumber(uid)].completed then
				beqlQ.party_quests[sender][tonumber(uid)].completed = tonumber(qstatus)
			end
		end
		return
	
	
	-- unknown request
	else
		return print("error for addonmsg: " ..msg)
	end
	


end

function beql:GetZones()
	beql.db.char.zones ={}
	
	local zones = {}
	local locale = GetLocale()
	
	SetMapToCurrentZone()
	local startzone = GetMapInfo()
	
	local canadd = false
	
	for i= -1, 5000 do
		SetMapByID(i)
		local zonename =  GetMapInfo();
		if zonename ~= startzone then
			canadd = true
		end
		
		if not zones[zonename] and canadd then
			zones[zonename] = i
		end
	end
	
	local tmpzones = {}
	
	for k,v in pairs(zones) do
		table.insert(tmpzones, k..v)
	end
	
	table.sort(tmpzones, function(a,b) return 
		string.gsub(a,"%a","")/1000 < string.gsub(b,"%a","")/1000 end)
	
	local zonestring = ""
	
	for k,v in pairs(tmpzones) do
		table.insert(beql.db.char.zones, {mapname = string.gsub(v,"%d+",""), mapid = string.gsub(v,"%a","") })
		zonestring = zonestring .. string.gsub(v,"%a","") .. " = " .. string.gsub(v,"%d+","") .. "__"
	end
	beql.db.char.zonestring = zonestring
end

function beql:SendChatMessage(...)
	beql.hooks.SendChatMessage(...)
	if (select(2, ...) == ("PARTY" or "RAID")) then
		print(...)
	end
end
-- EOF ---